@@ -3,10 +3,9 @@
 import numpy as np
 import sidpy
 from sidpy.base.num_utils import build_ind_val_matrices
-from scipy import fftpack
-from scipy.signal import hanning, blackman
+from scipy.signal.windows import hamming, blackman
 from skimage.transform import rescale
-
+import dask
 
 class ImageWindowing:
     """
@@ -31,7 +30,7 @@ class ImageWindowing:
             - 'interpol_factor' (float) (Optional, default is 1.0): Interpolation factor for windows to increase or decrease size of the windows.
             - 'zoom_factor' (integer or list of ints) (Optional, default is 1): Zoom the window by this factor, typically done for 'fft' mode to observe higher frequencies clearly
                             If passing a list of ints, this will determine the degree of cropping per axis
-            - 'filter' (string) (Optional, default is None): Filtering to use for the image window. Options are 'blackman', 'hanning'.
+            - 'filter' (string) (Optional, default is None): Filtering to use for the image window. Options are 'blackman', 'hamming'.
             The filter is applied to each window before 'mode'.
         - verbose : (Optional) Boolean
             Verbose flag. Default is False.
@@ -61,34 +60,34 @@ class ImageWindowing:
         if 'interpol_factor' in parms_dict.keys(): self.interpol_factor = parms_dict['interpol_factor']
         else:
             self.interpol_factor = 1
-            parms_dict['interpol_facor']=1
+            parms_dict['interpol_factor'] = 1
 
         if 'zoom_factor' in parms_dict.keys(): self.zoom_factor = parms_dict['zoom_factor']
         else:
             self.zoom_factor = 1
-            parms_dict['zoom_facor'] = 1
+            parms_dict['zoom_factor'] = 1
 
         # Based on the zoom and interpolation factors we need to figure out the final size of the window
         self.window_size_final_x, self.window_size_final_y = self._get_window_size()
         #Setup the filter for the window
-        self.filter = 'None'
-        self.filter_mat = np.ones((self.window_size_final_x, self.window_size_final_y))
+        if 'filter' in parms_dict.keys():
+            if parms_dict['filter'] not in ['blackman', 'hamming']:
+                raise ValueError("Parameter 'filter' must be one of 'hamming', 'blackman'")
+            else:
+                self.filter = parms_dict['filter']
+                if self.filter == 'hamming':
+                    filter_x = hamming(self.window_size_final_x)
+                    filter_y = hamming(self.window_size_final_y)
+                    self.filter_mat = np.sqrt(np.outer(filter_x, filter_y))
+                elif self.filter == 'blackman':
+                    filter_x = blackman(self.window_size_final_x)
+                    filter_y = blackman(self.window_size_final_y)
+                    self.filter_mat = np.sqrt(np.outer(filter_x, filter_y))
+        else:
+            self.filter = 'None'
 
         if self.mode=='fft':
             #load FFT options
-            if 'filter' in parms_dict.keys():
-                if parms_dict['filter'] not in ['blackman', 'hanning']:
-                    raise ValueError("Parameter 'filter' must be one of 'hanning', 'blackman'")
-                else:
-                    self.filter = parms_dict['filter']
-                    if self.filter=='hanning':
-                        filter_x = hanning(self.window_size_final_x)
-                        filter_y = hanning(self.window_size_final_y)
-                        self.filter_mat = np.sqrt(np.outer(filter_x,filter_y))
-                    elif self.filter=='blackman':
-                        filter_x = blackman(self.window_size_final_x)
-                        filter_y = blackman(self.window_size_final_y)
-                        self.filter_mat = np.sqrt(np.outer(filter_x,filter_y))
             if 'fft_mode' in parms_dict.keys():
                 if parms_dict['fft_mode'] not in ['abs', 'phase', 'complex']:
                     raise ValueError("Parameter 'fft_mode' must be \
@@ -111,12 +110,12 @@ class ImageWindowing:
         '''
 
         image_test = np.random.uniform(size=(self.window_size_x, self.window_size_y))
-        print('image test is shape {}'.format(image_test.shape))
+        #print('image test is shape {}'.format(image_test.shape))
         image_zoomed = self.zoom(image_test, self.zoom_factor)
-        print('image zoomed is shape {}'.format(image_zoomed.shape))
+        #print('image zoomed is shape {}'.format(image_zoomed.shape))
         #interpolate it
         zoomed_interpolated = rescale(image_zoomed, self.interpol_factor)
-        print('image zoomed interpol is shape {}'.format(zoomed_interpolated.shape))
+        #print('image zoomed interpol is shape {}'.format(zoomed_interpolated.shape))
         return zoomed_interpolated.shape[0],zoomed_interpolated.shape[1]
 
     def MakeWindows(self, dataset, dim_slice=None):
@@ -139,6 +138,7 @@ class ImageWindowing:
 
         # This is the windowing function. Will generate the windows (but not the FFT)
         num_dimensions = dataset.ndim
+        self.dataset = dataset
         if dim_slice is None:
             if num_dimensions > 2:
                 raise ValueError('You have specified windowing on a sidpy dataset '
@@ -184,22 +184,33 @@ class ImageWindowing:
         pca_mat = np.zeros(shape=(pos_vec.shape[0], np.prod(window_size_final)), dtype=np.complex64)
         pos_vec = np.int32(pos_vec)
 
-        for ind, pos in enumerate(pos_vec):
+        def make_windows_parallel(ind, pos):
             start_stop = [slice(x, x + y, 1) for x, y in zip(pos, window_size)]
             full_slice = image_source[tuple(start_stop)]
             full_slice = self._return_win_image_processed(full_slice)
-            pca_mat[ind] = full_slice.flatten()
+            full_slice_flat = full_slice.flatten()
+            return full_slice_flat
 
+        window_results = []
+        for ind, pos in enumerate(pos_vec):
+            lazy_result = dask.delayed(make_windows_parallel)(ind, pos)
+            window_results.append(lazy_result)
+
+        pca_mat = dask.compute(*window_results)
+        pca_mat = np.array(pca_mat) #it comes out as a tuple, make it array
+        
         self.pos_vec = pos_vec
 
         # Get the positions and make them dimensions
-        new_x_vals = np.linspace(dataset._axes[image_dims[0]].values.min(),
+        new_y_vals = np.linspace(dataset._axes[image_dims[0]].values.min(),
                                  dataset._axes[image_dims[0]].values.max(), len(np.unique(pos_vec[:, 0])))
 
-        new_y_vals = np.linspace(dataset._axes[image_dims[1]].values.min(),
+        new_x_vals = np.linspace(dataset._axes[image_dims[1]].values.min(),
                                  dataset._axes[image_dims[1]].values.max(), len(np.unique(pos_vec[:, 1])))
         if self.verbose:
-            print("position values x {} and y {}".format(new_x_vals, new_y_vals))
+            print("position values x {} and y {}".format(new_y_vals, new_x_vals))
+        
+        self.pca_mat = pca_mat
         windows_reshaped = pca_mat.reshape(len(new_x_vals), len(new_y_vals),
                                            self.window_size_final_x, self.window_size_final_y)
         if self.verbose:
@@ -234,11 +245,12 @@ class ImageWindowing:
 
         window_extent_y = (dataset._axes[image_dims[1]].values.max() -
                            dataset._axes[image_dims[1]].values.min()) * window_size_fraction_y
-
+        window_units = dataset._axes[image_dims[0]].units
         if self.mode =='fft':
             #to check if this is correct
             z_dimx = np.linspace(0, 1.0/(window_extent_x / self.zoom_factor), data_set.shape[2])
             z_dimy = np.linspace(0, 1.0/(window_extent_y / self.zoom_factor), data_set.shape[3])
+            window_units = dataset._axes[image_dims[0]].units + '^-1'
         else:
             z_dimx = np.linspace(0, window_extent_x/self.zoom_factor, data_set.shape[2])
             z_dimy = np.linspace(0, window_extent_y/self.zoom_factor, data_set.shape[3])
@@ -257,12 +269,12 @@ class ImageWindowing:
 
         data_set.set_dimension(2, sidpy.Dimension(z_dimx,
                                                   name='WindowX',
-                                                  units='m', quantity='kx',
+                                                  units=window_units, quantity='kx',
                                                   dimension_type='spectral'))
 
         data_set.set_dimension(3, sidpy.Dimension(z_dimy,
                                                   name='WindowY',
-                                                  units='m', quantity='ky',
+                                                  units=window_units, quantity='ky',
                                                   dimension_type='spectral'))
 
         # append metadata
@@ -273,8 +285,7 @@ class ImageWindowing:
     def _return_win_image_processed(self, img_window):
         #Real image slice, returns it back with image processed
 
-        if self.filter != 'None':
-            img_window *= self.filter_mat  # Apply filter
+
         if self.mode == 'fft': # Apply FFT if needed
             img_window = np.fft.fftshift(np.fft.fft2(img_window))
             if self.fft_mode == 'amp':
@@ -283,7 +294,7 @@ class ImageWindowing:
                 img_window = np.angle(img_window)
             elif self.fft_mode == 'complex':
                 img_window = np.array(img_window, dtype = np.complex64)
-        
+
         #Zoom and interpolate if needed
         if self.zoom_factor == 1 and self.interpol_factor == 1:
             return img_window
@@ -291,6 +302,9 @@ class ImageWindowing:
             img_window = self.zoom(img_window, self.zoom_factor)  # Zoom it
             img_window = self.rescale_win(img_window, self.interpol_factor)  # Rescale
 
+        if self.filter != 'None':
+            img_window *= self.filter_mat  # Apply filter
+
         return img_window
 
     def _merge_dictionaries(self, dict1, dict2):
@@ -330,6 +344,18 @@ class ImageWindowing:
             
         return complex_rescaled_image
 
+    def clean_image(self, n_comps = 4):
+        self.mode = 'image'
+        self.interpol_factor = 1
+        self.zoom_factor = 1
+        self.window_size_final_x = self.window_size_x
+        self.window_size_final_y = self.window_size_y
+        new_windows = self.MakeWindows(self.dataset)
+        
+        from ..learn.ml import MatrixFactor
+        svd_results = MatrixFactor(new_windows, method='svd', n_components = n_comps)
+
+        return svd_results
 
 
 
